;; RADICAL EMERGENCE MODEL

breed [techs tech] ;; instead of turtles, let's call them technologies
breed [developers developer]

techs-own [
 product ;; the "product" of the technology
 tech-uncertainty ;; +/- uncertainty of what the tech does, recalculated when more techs use this one
 perceived-product ;; the product as perceived by developers
 cost ;; the cost of the technology
 distance-to-nearest-target;; and a single variable with the minimum distance to a target
 nearest-target ;; allows the techs to be divided by the nearest target (for competition and fitness determination)
 components ;; a list of components used
 cost-of-components ;; cost of components used 
 contributor-tech-whos ;; a list of Whos - which unique techs are used by this tech
 who-uses-me ;; a list of Whos of techs that use this technology
 how-many-use-me ;; basically a length of who-uses-me list; put here to speed up the updates!
 created ;; timestamp when developed
 parent ;; [who] of parent
 last-updated ;; timestamp when updated last
 active-repertoire? ;; is tech in active use?
 invented-by ;; who invented me?
 tested-this-turn? ;; is this tried as a component?
 how-many-times-tested ;; how many times a tech has been tried as a component
 debug ;; general debug variable
]

developers-own [
  strategy ;; explorer or improver
]

globals [
  ;; for loading LHS sample - courtesy of Igor Nikolic
  lhsExperimentNumber ;; experiment numbber
  p1 ;; First parameter
  p2 ;; Second parameter
  p3 ;; 
  p4 ;; ...
  p5
  p6
  lhs ;; the file to hold the experiments
  
  experiment-number ;; used to save experiment number for LHS
  
  ;; Set in the Interface screen
  
  ;; kill-negative-techs? ;; not used in this version
  ;; debugging-mode? ;; when TRUE, prints what's happening
  ;; random-seed? ;; Whether to use random seed (TRUE) or seed 123456
  ;; stop-at-tick ;; when to stop
  ;; max-tech-level ;; highest target/valley values
  ;; valley-effects ;; whether the valleys affect entire technologies, or at component level (i.e. if "3" is a valley, 1+2 is not permitted)
  ;; density-of-targets ;; density of target values
  ;; density-of-valleys ;; and valleys
  ;; uncertainty-of-technology 
  ;; uncertainty-of-needs
  ;; learning-curve-tech
  ;; learning-curve-need
  
  list-of-operators
  list-of-target-values ;; lists all the target product values, or the peaks in the fitness landscape
  attempts-at-target ;; lists uncertainties associated with same position target; updated when tech variable nearest-target increases
  targets-reached-this-turn?;; TRUE/FALSE, are there targets created this turn?
  list-of-reached-targets-this-turn ;; list of targets reached this turn
  list-of-reached-targets ;; list of targets reached
  list-of-almost-reached-targets ;; list of targets techs are close by
  list-of-certain-targets ;; list of targets with uncertainty = 0
  target-value ;; target for current tick
  starting-tech ;; tech that is used as a starting point; the "parent"
  space-of-adjacent-possible ;; list of targets that neighbor already discovered targets; e.g. if list-of-target-values is [2 4 6 8] and target 4 is discovered, 2 and 6 are adjacent possibles
  uncertainty-of-adjacent-possible ;; uncertainties associated with those values
  list-of-valleys ;; list of "valley" values, ie. infeasible technologies
  list-of-maxed-techs ;; list of technologies with product = cost
  list-of-primitives ;; list of "primitive" technologies
  cumulative-replacements ;;  counting cumulative replacements
  list-of-replacements ;; a list where number of replacements is saved during update, to draw a histogram
  highest-tech-reached ;; the highest product reached so far 
  list-of-techlevels-found ;; a list of techlevels found
  size-of-active-repertoire ;; number of techs in active repertoire
  viable-techs-created-this-turn? ;; were there viable techs created during this turn? 
  list-of-products-created-this-turn ;; list of all products created this turn
  last-techlevel-created ;; these are used to update proper technologies 
  last-tech-who-created ;; these are used to update proper technologies
  who-of-last-tech-created
  stop-called? ;; TRUE/FALSE, mostly for debugging
  obsoleted-techs ;; used in update techs procedure
  obsolete-tech-who ;; used in the update techs procedure
  proceed-to-tech-creation?
  prospective-parent
  prospective-components
  prospective-operators 
  prospective-cost-of-components 
  prospective-cost
  prospective-contributor-tech-whos 
  prospective-product
  prospective-distance-to-nearest-target
  prospective-nearest-target  
  old-product ;; debugging variable...
  inventor
  use-saved-tgts?
  
  ;; related to data dump
  dump_ticks_internal
  tech_data_filename
  globals_data_filename
  settings_data_filename
  commonfilename
  savedirectory
  
  techs-tested-this-turn ;; a list of whos of techs that were used this turn as components
  max-components-to-add
  
  t_learn ;; learning curve variables
  n_learn
  
  seed-number ;; random seed number
]

to setup-parameters
  ;; this methods is to be run by the Behaviour Space, which sets the lhsExperimentNumber variable
  ;; from 1 to n
  let currentExperiment item experiment-number lhs 
  set p1 item  0 currentExperiment
  set p2 item  1 currentExperiment
  set p3 item  2 currentExperiment
  set p4 item  3 currentExperiment
  set p5 item  4 currentExperiment
  set p6 item  5 currentExperiment

  set density-of-targets p1
  set density-of-valleys p2
  set uncertainty-of-technology p3
  set uncertainty-of-needs p4
  set learning-curve-tech p5
  set learning-curve-need p6
end

to load-experiment-number
  set-current-directory "/home/jooliesays/JoolieKasmire/AdderExperiments/PeakSeek" 
  set experiment-number []
  file-open "experiment-number.txt"
  set experiment-number file-read
  file-close
end

to save-experiment-number
  set-current-directory "/home/jooliesays/JoolieKasmire/AdderExperiments/PeakSeek"
  file-delete "experiment-number.txt"
  file-open "experiment-number.txt"
  file-write (experiment-number + 1) 
  file-close
end

to reset-experiment-number
  set-current-directory "/home/jooliesays/JoolieKasmire/AdderExperiments/PeakSeek"
  file-delete "experiment-number.txt"
  file-open "experiment-number.txt"
  file-write 0
  file-close
end  




to setup
  clear-all
  no-display
  set seed-number seed
  if random-seed? = TRUE [set seed-number random 100000]
  random-seed seed-number
  set savedirectory "/home/jooliesays/JoolieKasmire/AdderExperiments/PeakSeek"
  
  
  set list-of-target-values [] ;; setting up an empty list
  set attempts-at-target []
  set list-of-reached-targets []
  set list-of-almost-reached-targets []
  set list-of-certain-targets []
  set list-of-valleys []
  set space-of-adjacent-possible []
  set uncertainty-of-adjacent-possible []
  set list-of-maxed-techs []
  set highest-tech-reached 1
  set size-of-active-repertoire 1 
  set list-of-techlevels-found [1]
  
  set max-components-to-add 10 ;; max number of components to add at any one time
  
  set t_learn -1 * learning-curve-tech / 100 / log 2 10 ;; these come from learning curve literature
  set n_learn -1 * learning-curve-need / 100 / log 2 10 ;; and are exponents for learning curve calculation
  
 
  
  create-primitives 

  ;; creating developer agents
  create-developers 1 [
    set strategy "innovator"
  ]
 
 
  reset-ticks
end


to create-primitives
  ;; CREATING A SET OF PRIMITIVES
  ;;
  create-techs 1 [
    set product 1
    set tech-uncertainty 1 * uncertainty-of-technology / 100
    
    let uncertainty-low-bound 1 - tech-uncertainty
    let uncertainty-high-bound 1 + tech-uncertainty
    let uncertainty-range uncertainty-high-bound - uncertainty-low-bound
    set perceived-product ((((random 101) / 100) * uncertainty-range) + uncertainty-low-bound) 
    
    set cost 1
    set components [1] ;; sets up an empty list
    set cost-of-components [1] ;; empty, not needed usually but here to avoid bugs
    set contributor-tech-whos [0] ;; empty, not needed usually but here to avoid bugs
    set who-uses-me []
    set how-many-use-me 0 
    set distance-to-nearest-target 0
    set nearest-target 0
    set created 0 ;; primitives are gen 0
    set active-repertoire? TRUE
    set list-of-primitives [product] of techs ;; this is needed in setting targets, to avoid primitives being targets
    set how-many-times-tested 0
  ]
end

to create-targets-and-valleys
  
  ;; creating targets and valleys
  
  let max-random-tech-level max-tech-level - 1
  
  while [length list-of-target-values <= ((max-tech-level * (density-of-targets / 100)) - 1)] [ ;; -1 because it starts from 0
    set list-of-target-values lput ((random max-random-tech-level) + 2) list-of-target-values ;; +2 to prevent targets being 0 or 1
    set list-of-target-values remove-duplicates list-of-target-values ;; avoiding duplicates
  ]
  set list-of-target-values sort list-of-target-values ;; ... and sorting
  
                                                       ;; randomizing uncertainties
  
  set attempts-at-target map random list-of-target-values
  set attempts-at-target map [0 * ?] attempts-at-target ;; sets up zeros only
  
                                                        ;; setting up initial adjacent possible - first target in the list
  let adjacent 0
  while [adjacent < neighborhood-size] [ 
    set space-of-adjacent-possible lput item adjacent list-of-target-values space-of-adjacent-possible
    set uncertainty-of-adjacent-possible lput item adjacent attempts-at-target uncertainty-of-adjacent-possible
    set adjacent adjacent + 1
    print(word "Space of adjacent possible: " length space-of-adjacent-possible)
  ]
  if infinite-neighborhood? = TRUE [
    set space-of-adjacent-possible list-of-target-values
    set uncertainty-of-adjacent-possible attempts-at-target]
  
  ;; Creating valleys
  
  if density-of-targets + density-of-valleys >= 100 [set density-of-valleys 100 - density-of-targets] ;; Making sure that the number of valleys is not higher than max tech level
  
  if density-of-targets < 100 [ ;; no sense trying to create valleys if everything's a target!
    let number-of-valleys (max-tech-level * (density-of-valleys / 100) - 1)
    
    ;; previous version tried to randomly "fill in" the list of valleys
    ;; this version selects randomly from what's left
    ;; to avoid spending infinite time chasing that last number in situations where density of targets + density of valleys -> 100
    let potential-valleys []
    let number 1
    while [length potential-valleys <= max-tech-level] [ ; create a list from 1 to max-tech-level
      set potential-valleys lput number potential-valleys
      set number number + 1
    ]

    set potential-valleys remove list-of-target-values potential-valleys ;; then remove targets from the list
    set list-of-valleys n-of number-of-valleys potential-valleys ;; then randomly pick the valleys
    set list-of-valleys sort list-of-valleys
  ]
end

to save-targets-and-valleys  
  set-current-directory (word savedirectory "/saved_seeds")
  let targetsname (word seed-number "_dT" density-of-targets "_dV" density-of-valleys "_saved_seed_and_targets.txt")
  let valleysname (word seed-number "_dT" density-of-targets "_dV" density-of-valleys "_saved_valleys.txt")
  
  if file-exists? targetsname [file-delete targetsname]
  if file-exists? valleysname [file-delete valleysname]
  file-open targetsname
  file-print seed-number
  foreach list-of-target-values [
    file-print ?
  ]
  file-close
  file-open valleysname
  foreach list-of-valleys [
    file-print ?
  ]
  file-close
end

to load-targets-and-valleys
  set-current-directory (word savedirectory "/saved_seeds")
  
  let targetsname (word seed-number "_dT" density-of-targets "_dV" density-of-valleys "_saved_seed_and_targets.txt")
  let valleysname (word seed-number "_dT" density-of-targets "_dV" density-of-valleys "_saved_valleys.txt")

  set list-of-target-values []
  file-open targetsname
  set seed-number file-read
  random-seed seed-number
  while [file-at-end? = FALSE] [
     set list-of-target-values lput file-read list-of-target-values
  ]
  file-close
  
  set list-of-valleys []
  file-open valleysname
  while [file-at-end? = FALSE] [
    set list-of-valleys lput file-read list-of-valleys
  ]
  file-close

  set attempts-at-target map random list-of-target-values
  set attempts-at-target map [0 * ?] attempts-at-target ;; sets up zeros only
  
  ;; setting up initial adjacent possible - first target in the list
  
  set space-of-adjacent-possible lput first list-of-target-values space-of-adjacent-possible
  set uncertainty-of-adjacent-possible lput first attempts-at-target uncertainty-of-adjacent-possible
  if infinite-neighborhood? = TRUE [
    set space-of-adjacent-possible list-of-target-values
    set uncertainty-of-adjacent-possible attempts-at-target]
end



to setup-data-dump
  ;; DATA DUMP SETUP
  
  set-current-directory savedirectory
  
  if dump_every_turn? = FALSE [set dump_ticks_internal read-from-string dump_ticks] ;; so that we get a list
                                                                                    ;; saved with timestamp
  let hours substring date-and-time 0 2
  let mins substring date-and-time 3 5
  let seconds substring date-and-time 6 8
  let date substring date-and-time 16 27
  
  set commonfilename (word ;; common prefix for files created in the data dump
    "learn20_"
    seed-number "_"
    "dT" density-of-targets 
    "_dV" density-of-valleys 
    "_Ut" uncertainty-of-technology 
    "_Un" uncertainty-of-needs) ;; final line: identifier for simulation type
  
  set tech_data_filename (word commonfilename "techdata.csv")
  let settings_dump_filename (word commonfilename "settings.csv")
  set globals_data_filename (word commonfilename "globalsdata.csv")
  ;; so that we get 3 files, one for settings, one for data, and one for globals easier for R and other programs esp. when there are thousands of valleys and targets
   
  
  if data_dump? = TRUE [
    ;; let's dump the settings first
    file-open settings_dump_filename
    file-print (word "Random seed: " seed-number)
    file-print (word "Max tech level: " max-tech-level)
    file-print (word "Kill negative techs? " kill-negative-techs?)
    file-print (word "Valley effects: " valley_effects)
    file-print("targets,valleys")
    let temporary-list-of-targets list-of-target-values ;; we'll create temporary variables to help printing
    let temporary-list-of-valleys list-of-valleys
    while [not empty? temporary-list-of-targets OR not empty? temporary-list-of-valleys] [
      let next-target-value 0
      let next-valley-value 0
      ifelse not empty? temporary-list-of-targets [set next-target-value item 0 temporary-list-of-targets] [set next-target-value "NA"]
      ifelse not empty? temporary-list-of-valleys [set next-valley-value item 0 temporary-list-of-valleys] [set next-valley-value "NA"]
      file-print(word next-target-value "," next-valley-value)
      if not empty? temporary-list-of-targets [set temporary-list-of-targets remove-item 0 temporary-list-of-targets]
      if not empty? temporary-list-of-valleys [set temporary-list-of-valleys remove-item 0 temporary-list-of-valleys]
    ]
    
    file-close
    print settings_dump_filename
    
    ;; then create a file for tech data dump
    file-open tech_data_filename
    file-print("who,parent,product,created,cost,contributor-tech-whos,nearest-tgt,who-uses-me,how-many-use-me,times-tested,tick") ;; creates the labels at line 1, remember to edit if you change the content of the dump!
    file-close
    print tech_data_filename
    
    ;; and for globals
    
    file-open globals_data_filename
    file-print("list-of-reached-targets,list-of-almost-reached-targets,list-of-maxed-techs,list-of-replacements,space-of-adjacent-possible,target-value,starting-tech,highest-tech-reached,targets-reached-this-turn?,attempts-at-target,tick")
    file-close
    print globals_data_filename
  ]
end



to go
  if debugging-mode? = TRUE [print (word " " )]
  if ticks >= stop-at-tick [
    if data_dump? = TRUE [final-turn-data-dump]
    stop]
  tick
  if debugging-mode? = TRUE [print(word"--- TICK " ticks " START ---")]
  set viable-techs-created-this-turn? FALSE ;; no techs created yet
  set targets-reached-this-turn? FALSE ;; no targets reached either
  set list-of-products-created-this-turn []
  set techs-tested-this-turn []
  set list-of-reached-targets-this-turn []
  
  ;; Ask developers to create new technologies from techs existing at the start of the turn
  if debugging-mode? = TRUE [print(word"--- INNOVATORS CREATE TECH ---")]
  ask developers with [strategy = "innovator"] [incrementally-create-new-tech]
  if debugging-mode? = TRUE [print(word"--- COST LEADERS IMPROVE ---")]
  ask developers with [strategy = "cost_leader"] [improve-existing-tech]
  
  if debugging-mode? = TRUE [
    print(word "--- ADJACENT POSSIBLE UPDATE ---")
    print(word "Targets reached? " targets-reached-this-turn?)
  ]
  ;; update adjacent possible
  if infinite-neighborhood? = FALSE [update-adjacent-possible]
  
  ;; update target list
  
  if targets-reached-this-turn? = TRUE [
    set list-of-reached-targets sentence remove-duplicates list-of-reached-targets-this-turn list-of-reached-targets
  ]
  
  if debugging-mode? = TRUE [
    print (word "List of reached targets:")
    print list-of-reached-targets
  ]
  
  adjust-tech-uncertainty  
  
  if data_dump? = TRUE [ ;; data dump sequence
    ifelse dump_every_turn? = TRUE [dump-data] [ ;; checks if data is to be dumped every turn, or at specified ticks
      if not empty? dump_ticks_internal and ticks = item 0 dump_ticks_internal [ ;; specified ticks
        dump-data
        set dump_ticks_internal remove-item 0 dump_ticks_internal
      ]]
    ]
  
end

to adjust-tech-uncertainty
   if length techs-tested-this-turn > 0 [
    if debugging-mode? = TRUE [
      print (word "--- TECH UNCERTAINTY ADJUSTMENTS ---")
      ]
    foreach remove-duplicates techs-tested-this-turn [ask tech ? [
      set tech-uncertainty (precision (product * ((uncertainty-of-technology / 100) * (how-many-times-tested + 1) ^ t_learn)) 1)
      ;; if only use improves knowledge, change how-many-times-tested + 1 to how-many-use-me
      if debugging-mode? = TRUE [
        print (word "Tech: " who " P: " product " Unc: " tech-uncertainty)
      ]
  ]]]
end

to  load-experiments ;; courtesy of Igor Nikolic
  set-current-directory "/home/jooliesays/JoolieKasmire/AdderExperiments/PeakSeek"   
  ;; We check to make sure the file exists first
  ifelse(file-exists? "latin_hypercube_sample_1.data" )
  [
    ;; We are saving the data into a matrix, so it only needs to be loaded once.
  set lhs []
  file-open "latin_hypercube_sample_1.data"
    ;; Read in all the data in the file
    while [ not file-at-end? ]
    [
      ;; file-read gives you variables.  In this case numbers and booleans
      ;; We store them in a double list (ex [[1 1 9.9999] [1 2 9.9999] ...
      ;; Each iteration we append the next sex-tuple to the current list
      set lhs sentence lhs (list (list file-read file-read file-read file-read file-read file-read))
    ]
   ;; Done reading in patch information.  Close the file.
    file-close
  ]
  [ user-message "There is no File latin_hypercube_sample_1.data file in current directory!" ]
  
end



to incrementally-create-new-tech
  
  ;; select target
  if debugging-mode? = TRUE [print (word"--- DEVELOPER " [who] of self " PICKS A TARGET ---")]
  if not empty? space-of-adjacent-possible [
    set target-value one-of space-of-adjacent-possible]
  if empty? space-of-adjacent-possible and not empty? list-of-almost-reached-targets [
    set target-value one-of list-of-almost-reached-targets]
  if empty? space-of-adjacent-possible and empty? list-of-almost-reached-targets [
   stop]
  let target-position position target-value list-of-target-values
  let target-attempts item target-position attempts-at-target
  let target-uncertainty (target-value * uncertainty-of-needs / 100) * ((target-attempts + 1) ^ n_learn)
  
  ;; target-attempts and how-many-use-me need to be at least 1 or otherwise learning curve calculation results to infinity!
  
  set attempts-at-target replace-item target-position attempts-at-target (target-attempts + 1) ;; now the target will be tried again, and we'll learn something
  
  let target-low-bound target-value - target-uncertainty
  if target-low-bound < 0 [set target-low-bound 0]
  let target-high-bound target-value + target-uncertainty
  let target-range target-high-bound - target-low-bound
  
  ;; let perceived-target ((((random 101) / 100) * target-range) + target-low-bound) ;; randomizing element might not be really necessary, one could use just target-low-bound for target
  let perceived-target precision target-low-bound 1
  
  
  if debugging-mode? = TRUE [
    print(word "Target: " target-value ", attempts: " target-attempts ", unc.: " (precision target-uncertainty 1)", low: " target-low-bound ", high: " target-high-bound ", g_p: " perceived-target)
  ]
  
  ;; determine starting tech for incremental improvement
  
  let not-maxed-techs techs with [length components < max-components] ;; first, eliminate techs that have too much components
  ;; the idea behind next line: select techs whose product - tech uncertainty is less than target high bound, i.e. there is a chance it can be used as a starting point
  ;; the reason: this way we can use simply adding (+)
  let starting-techs not-maxed-techs with [(product + (precision tech-uncertainty 1)) < perceived-target ] ;; i.e. there is no chance it could be the target
  if count starting-techs = 0 [
    set starting-techs not-maxed-techs with-min [product + (precision tech-uncertainty 1)]
  ]
  
  set starting-tech one-of starting-techs with-max [product - (precision tech-uncertainty 1)];; picks the closest to target one
  
  
  if debugging-mode? = TRUE [
    print(word "Starting techs " starting-techs)  
    print(word "P of start-techs: " [product] of starting-techs)
  ]
  

  
  ;; determine perceived distance
  
  let starting-point [product] of starting-tech + (precision ([tech-uncertainty] of starting-tech) 1)
  let perceived-distance (precision (perceived-target - starting-point) 1)
  
  if debugging-mode? = TRUE [
    print (word "Who of starting tech: " [who] of starting-tech " P: " [product] of starting-tech " P_unc: "  [tech-uncertainty] of starting-tech)
    print (word "D^p_min: " perceived-distance " start point: " starting-point)
  ]
  
  set prospective-parent [who] of starting-tech ;; for parentage
  set prospective-components [components] of starting-tech
  set prospective-contributor-tech-whos [contributor-tech-whos] of starting-tech
  set prospective-cost-of-components [cost-of-components] of starting-tech
  set techs-tested-this-turn lput [who] of starting-tech techs-tested-this-turn
  ask starting-tech [set how-many-times-tested how-many-times-tested + 1]
  
  if debugging-mode? = TRUE [print(word "Starting components: " prospective-components)]
  
  ;; loop for adding new components
  let components-added 0
  let smallest-component techs with [active-repertoire? = TRUE]
  set smallest-component one-of smallest-component with-min [product]
  let smallest-increment ([product] of smallest-component - ( precision ([tech-uncertainty] of smallest-component) 1))
  let stop-search? FALSE
  
  if perceived-distance < smallest-increment [ ;; this is needed to avoid cases where perceived distance is less than 1
    set perceived-distance smallest-increment
    if debugging-mode? = TRUE [print (word "Perceived distance set to smallest increment!")]
    ] 
  
  while [perceived-distance >= smallest-increment and length prospective-components < max-components and components-added <= max-components-to-add and stop-search? = FALSE] [
    
    ask techs [set tested-this-turn? FALSE]
    if debugging-mode? = TRUE [
      print (word "Add component " (components-added + 1))
      print (word "Perceived distance before selection: " perceived-distance)
    ]
    let potential-components techs with [(perceived-distance - product + tech-uncertainty) >= 0] ;; Want to avoid having techs with negative distance
    if count potential-components = 0 [set potential-components techs with-min [product + tech-uncertainty]] ;; in case there are no non-negative techs, picks the smallest component
    if debugging-mode? = TRUE [
      print (word "Potential components: " potential-components)
      print (word "P's of potential components: " ([product] of potential-components with [tested-this-turn? = FALSE]))
      ]
    
    if valley_effects = "product-level" [
      let chosen-component one-of potential-components with-min [perceived-distance - product - tech-uncertainty] ;; selects one tech that gets closest to the perceived target
      
      
      
      ask chosen-component [
        set tested-this-turn? TRUE
        set how-many-times-tested how-many-times-tested + 1
        ]
      
      if debugging-mode? = TRUE [
        print (word "Select: " chosen-component ", P: " [product] of chosen-component ", select criteria: " (perceived-distance - ([product] of chosen-component - [tech-uncertainty] of chosen-component)))
      ]
      
      
      
      set perceived-distance perceived-distance - [product] of chosen-component + (precision ([tech-uncertainty] of chosen-component) 1);; now the new tech is closer to the perceived target
      if debugging-mode? = TRUE [print (word "D_p after selection: " perceived-distance)]
      
      set prospective-components lput [product] of chosen-component prospective-components ;; the list of components is updated...
      if debugging-mode? = TRUE [print(word "Components after selection: " prospective-components)]
      
      set prospective-contributor-tech-whos lput [who] of chosen-component prospective-contributor-tech-whos ;; ... as is the ID list
      set prospective-cost-of-components lput [cost] of chosen-component prospective-cost-of-components ;; and the cost
      set techs-tested-this-turn lput [who] of chosen-component techs-tested-this-turn
      set components-added components-added + 1
    ]
    
    
    if valley_effects = "component-level" [ ;; component level valley effects mean that at no point during the "assembly" may the partially finished product have valley value
      let search-over? FALSE
      while [search-over? = FALSE] [
        set potential-components potential-components with [tested-this-turn? = FALSE]
        let chosen-component one-of potential-components with-min [perceived-distance - product - (precision tech-uncertainty 1)] ;; selects one tech that gets closest to the perceived target
        
        ask chosen-component [  
          set tested-this-turn? TRUE ;; this is needed so that same tech is not considered twice
          set how-many-times-tested how-many-times-tested + 1 ;; and this indicates that the tech has been tested one more time
          ] 
        
        set techs-tested-this-turn lput [who] of chosen-component techs-tested-this-turn
        let component-assembly sum prospective-components
        set component-assembly component-assembly + [product] of chosen-component
        if not member? component-assembly list-of-valleys [
          if debugging-mode? = TRUE [
            print (word "Select: " chosen-component ", P: " [product] of chosen-component ", select criteria: " (precision (perceived-distance - ([product] of chosen-component - [tech-uncertainty] of chosen-component)) 1 ))
          ]
      
          set perceived-distance perceived-distance - [product] of chosen-component + precision ([tech-uncertainty] of chosen-component) 1;; now the new tech is closer to the perceived target
          if debugging-mode? = TRUE [print (word "D_p after selection: " perceived-distance)]
      
          set prospective-components lput [product] of chosen-component prospective-components ;; the list of components is updated...
          if debugging-mode? = TRUE [print(word "Components after selection: " prospective-components)]
      
          set prospective-contributor-tech-whos lput [who] of chosen-component prospective-contributor-tech-whos ;; ... as is the ID list
          set prospective-cost-of-components lput [cost] of chosen-component prospective-cost-of-components ;; and the cost
          set components-added components-added + 1
          set search-over? TRUE
        ]
        
        if member? component-assembly list-of-valleys [
           if debugging-mode? = TRUE [
             print (word "Component assembly " component-assembly " unfeasible")
             print (word "Potential components left: " (count potential-components with [tested-this-turn? = FALSE]))
           ]
           let number-of-potentials count potential-components with [tested-this-turn? = FALSE]
           if number-of-potentials = 0 [
             set search-over? TRUE
             set stop-search? TRUE
             
             if debugging-mode? = TRUE [
               print (word "No more possible components; this list should be empty: " potential-components)
               print (word "Count: " count potential-components with [tested-this-turn? = FALSE])
               print (word "number-of-potentials: " number-of-potentials)
               print (word "Techs: " ([who] of potential-components))
             ]
           ]
        ]
      ]
    ]        
  ]
  
  ;; determine product
  set prospective-product sum prospective-components
  
  ;; determine cost
  set prospective-cost sum prospective-cost-of-components
 
  ;; determining nearest target
  let distance-to-target-product-values (map [( ? - prospective-product  ) ] list-of-target-values )
  let distance-to-target-absolute-values map abs distance-to-target-product-values ;; this complication is needed so that we can extract the minimum absolute value
  let position-of-distance-value position min distance-to-target-absolute-values distance-to-target-absolute-values ;; ... first we note the position of the lowest value (if tied, the first is picked?)
  set prospective-nearest-target item position-of-distance-value list-of-target-values ;; then we extract that value from the list of target values
  set prospective-distance-to-nearest-target item position-of-distance-value distance-to-target-product-values ;; and also the distance value
  
  if debugging-mode? = TRUE [
    print (word "Comps.: " prospective-components)
    print (word "Contributors: " prospective-contributor-tech-whos)
    print (word "Product: " prospective-product)
  ]
  
  set viable-techs-created-this-turn? TRUE
  set proceed-to-tech-creation? TRUE
  
  check-viability-of-tech
  
end

to improve-existing-tech
end
 
 
 
;; routines
;; ---------------------------------------------


to check-viability-of-tech
  
  if prospective-product > max-tech-level [ ;; 
    set viable-techs-created-this-turn? FALSE
    set proceed-to-tech-creation? FALSE
   ] 
  
  if member? prospective-product list-of-valleys [ ;; test if the tech is a "valley" of infeasible designs
    set viable-techs-created-this-turn? FALSE
    set proceed-to-tech-creation? FALSE
    ]
  
  ;if member? prospective-product list-of-maxed-techs [ ;; a tech is not viable if it's tech level is already at maximum efficiency (cost = product)
  ;  set viable-techs-created-this-turn? FALSE
  ;  set proceed-to-tech-creation? FALSE
  ;  ]
  
  ;if not member? prospective-product list-of-reached-targets [ ;; a tech is not viable if a target is reached and the new tech is more expensive than the existing one
  ;  if count techs with [product = prospective-product] > 0 [
  ;    if prospective-cost < min [cost] of techs with [product = prospective-product] [
  ;      set viable-techs-created-this-turn? FALSE
  ;      set proceed-to-tech-creation? FALSE
  ;    ]
  ;  ]
  ; ]
  
  if member? prospective-product prospective-components [ ;; trying to avoid techs that are "circular," ie. whose product is same as one of the component's
    set viable-techs-created-this-turn? FALSE 
    set proceed-to-tech-creation? FALSE
    ]
  
  ;if not member? prospective-product list-of-techlevels-found [
  ;  if count techs with [product = prospective-product] > 0 [
  ;    if prospective-cost < min [cost] of techs with [product = prospective-product] [
  ;      set viable-techs-created-this-turn? TRUE
  ;      set proceed-to-tech-creation? TRUE
  ;    ]
  ;  ]
  ;]
  
  if member? prospective-product list-of-techlevels-found [ ;; eliminates technologies that are identical to existing ones
    if any? techs with [product = prospective-product and components = prospective-components] [
      set viable-techs-created-this-turn? FALSE
      set proceed-to-tech-creation? FALSE
    ]
  ]
  
  if proceed-to-tech-creation? = TRUE [
    set inventor [who] of self
    if debugging-mode? = TRUE [print(word"--- VIABILITY CHECK OK ---")]
    create-next-tech
    ]
  
  if debugging-mode? = TRUE and proceed-to-tech-creation? = FALSE [print(word"--- VIABILITY CHECK FAILED ---")]
  
end
  
 

to create-next-tech
  hatch-techs 1 [ ;; If all is OK, only then we'll create the technology
    if debugging-mode? = TRUE [print(word"--- TECH CREATION ---")]
    set color black ;; eye candy
    set created ticks ;; assigns timestamp
    set active-repertoire? FALSE ;; just in case
    set components prospective-components
    set cost-of-components prospective-cost-of-components
    set contributor-tech-whos prospective-contributor-tech-whos
    set who-uses-me [] ;; empty list for descendants
    set how-many-use-me 0
    set parent prospective-parent
    set product prospective-product
    set tech-uncertainty (prospective-product * uncertainty-of-technology / 100)
    set cost prospective-cost
    set invented-by inventor
    set viable-techs-created-this-turn? TRUE
    set last-techlevel-created product
    set who-of-last-tech-created [who] of self
    set list-of-techlevels-found lput [product] of self list-of-techlevels-found
    set list-of-products-created-this-turn lput product list-of-products-created-this-turn
    
    set how-many-times-tested 0
    
    ;; set techs-tested-this-turn sentence prospective-contributor-tech-whos techs-tested-this-turn
    if debugging-mode? = TRUE [print (word "Techs used this turn: " techs-tested-this-turn)]
  
    ;; SETTING DISTANCE TO TARGET AND UPDATING LISTS OF REACHED & CLOSEST TARGETS
    
    set distance-to-nearest-target prospective-distance-to-nearest-target
    set nearest-target prospective-nearest-target
    
    if not member? [nearest-target] of self list-of-almost-reached-targets and not member? [nearest-target] of self list-of-reached-targets [ ;; puts the nearest target to the list of almost reached targets
      set list-of-almost-reached-targets lput [nearest-target] of self list-of-almost-reached-targets
    ]
    
    ;; set list-of-reached-targets 
    if distance-to-nearest-target = 0 and not member? [product] of self list-of-reached-targets [
      set list-of-reached-targets-this-turn lput [product] of self list-of-reached-targets-this-turn
      set list-of-almost-reached-targets remove [product] of self list-of-almost-reached-targets 
      set targets-reached-this-turn? TRUE
      if debugging-mode? = TRUE [print (word self " reached target " [product] of self)]
    ]
    
    ;; Checking if this tech is now maxed out in efficiency (product = cost)
    if abs [product] of self = [cost] of self [
      set list-of-maxed-techs lput [product] of self list-of-maxed-techs
    ]
    
    ;; ADDING OWN [WHO] TO COMPONENT TECH'S LIST WHO-USES-ME (SETTING WHO-USES-ME LISTS)
    let my-own-who [who] of self
    foreach contributor-tech-whos [
      if ? != "na" [ask tech ? [
          set who-uses-me lput my-own-who who-uses-me
          set how-many-use-me how-many-use-me + 1
          ]]
    ]
    
    ;; CHECKING IF TECHNOLOGY FRONTIER ADVANCES
    
    if [product] of self > highest-tech-reached [ 
      set highest-tech-reached [product] of self ;; updates highest-tech-reached
      set list-of-techlevels-found lput [product] of self list-of-techlevels-found ;; adds the tech to the list of techs found
      if debugging-mode? = TRUE [print (word "At tick " ticks ", " list-of-techlevels-found)] ;; debugging...
    ]

  ]
end


to update-adjacent-possible
  ;; update space of adjacent possible (targets that neighbor already discovered targets, but are not discovered yet)
  if targets-reached-this-turn? = TRUE [ 
    foreach remove-duplicates list-of-reached-targets-this-turn [ ;; let's look through the results of this turn, shall we?
      if debugging-mode? = TRUE [print (word "Target found this turn: " ?)]
      if not member? ? list-of-reached-targets [ ;;The SAP needs to be updated ONLY if this is the first time this target is reached; 
        ;; the list of reached targets is updated AFTER SAP update in the Go procedure
        let reached-position position ? list-of-target-values ;; finds the position of last created tech's target from the list of target values
  
        let nth-neighbor 1 ;; starts from the next neighbor of the target value
        
        repeat neighborhood-size [
          if (reached-position + nth-neighbor) < length list-of-target-values [ ;; want to avoid going over the index as it would cause an error
            let adjacent-positive item (reached-position + nth-neighbor) list-of-target-values ;; finds nth higher neighbor          
            if not member? adjacent-positive space-of-adjacent-possible and not member? adjacent-positive list-of-reached-targets [ ;; if it's not already in the list, or a found target...
              set space-of-adjacent-possible lput adjacent-positive space-of-adjacent-possible ;; adds it to the list,
            ]
          ]
          if (reached-position - nth-neighbor) >= 0 [; want to avoid referring to index < 0 as it would cause an error
            let adjacent-negative item (reached-position - nth-neighbor) list-of-target-values ;; finds preceding adjacent possible
            if not member? adjacent-negative space-of-adjacent-possible and not member? adjacent-negative list-of-reached-targets [ ;; if it's not already in the list, or a found target...
              set space-of-adjacent-possible lput adjacent-negative space-of-adjacent-possible ;; adds it there
            ]
          ]
          set nth-neighbor nth-neighbor + 1 ;; advances the counter
        ]
        
        if member? ? space-of-adjacent-possible [ ;; if the target happens to be in the space of adjanced possible, removes it from the adjacent possible list
            set space-of-adjacent-possible remove ? space-of-adjacent-possible 
        ]
      ]
    ]
  ]   
  if debugging-mode? = TRUE [print (word "Adj. Poss.: " space-of-adjacent-possible)]
end


to dump-data
  if debugging-mode? = TRUE [print(word "Data dump, tick " ticks)]
  set-current-directory savedirectory
  
  ;; TECH DATA
  ;; data to be dumped (check at Setup if you make changes):
  ;; who,parent,product,created,cost,contributor-tech-whos,nearest-tgt,who-uses-me,how-many-use-me,times-tested,tick ;; creates the labels at line 1, remember to edit if you change the content of the dump!
  
  file-open tech_data_filename
  ask techs [
    file-print(word [who] of self "," 
      [parent] of self "," 
      [product] of self "," 
      [created] of self "," 
      [cost] of self "," 
      [contributor-tech-whos] of self  "," 
      [nearest-target] of self "," 
      [who-uses-me] of self "," 
      [how-many-use-me] of self "," 
      [how-many-times-tested] of self "," 
      ticks)
  ]
  file-close
  
  ;; GLOBALS DATA
  ;; data to be dumped (check at Setup if you make any changes):
  ;; "list-of-reached-targets,list-of-almost-reached-targets,list-of-maxed-techs,list-of-replacements,space-of-adjacent-possible,target-value,starting-tech,highest-tech-reached,targets-reached-this-turn?,attempts-at-target,tick")
  file-open globals_data_filename
  file-print(word list-of-reached-targets "," 
    list-of-almost-reached-targets "," 
    list-of-maxed-techs ","
    list-of-replacements ","
    space-of-adjacent-possible ","
    target-value ","
    [who] of starting-tech ","
    highest-tech-reached ","
    targets-reached-this-turn? ","
    attempts-at-target ","
    ticks)
  file-close  
end

to final-turn-data-dump
  set-current-directory savedirectory
  let final-network-filename (word commonfilename "tick_" ticks "_final_network.csv")
  file-open final-network-filename
  file-print "source,relationship,target,IA_ID"
  let IA_ID 0 ;; ID key to identify relationships & help later data analysis
  ask techs [
    let users [who-uses-me] of self
    file-print (word [parent] of self ",parent," [who] of self "," IA_ID)
    set IA_ID IA_ID + 1
    foreach users [
      file-print (word [who] of self ",component," ? "," IA_ID)
      set IA_ID IA_ID + 1  
    ]
  ]
  file-close
  
  let final-list-filename (word commonfilename "tick_" ticks "_final_list.csv")
  file-open final-list-filename
  file-print "who,product,cost,created,nearest-target"
  ask techs [
    file-print(word [who] of self "," [product] of self "," [cost] of self "," [created] of self "," [nearest-target] of self)
    ]
  file-close
    
end
    
    
  
@#$#@#$#@
GRAPHICS-WINDOW
700
375
945
619
0
0
213.0
1
10
1
1
1
0
1
1
1
0
0
0
0
1
1
1
ticks
30.0

BUTTON
390
115
515
148
Normal setup
setup\ncreate-targets-and-valleys\nsave-targets-and-valleys\nsetup-data-dump
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
615
115
700
148
Go once
go
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SLIDER
5
185
180
218
max-components
max-components
2
50
50
1
1
NIL
HORIZONTAL

SWITCH
5
10
135
43
kill-negative-techs?
kill-negative-techs?
0
1
-1000

MONITOR
100
380
190
425
Top tech
max [product] of techs
17
1
11

MONITOR
560
380
660
425
Replacements
sum list-of-replacements
17
1
11

MONITOR
5
465
95
510
Active
size-of-active-repertoire
17
1
11

BUTTON
525
115
610
190
Go
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

MONITOR
5
415
95
460
N:o of techs
count techs
17
1
11

SWITCH
140
10
307
43
debugging-mode?
debugging-mode?
1
1
-1000

INPUTBOX
5
45
135
105
max-tech-level
1000
1
0
Number

SLIDER
5
115
180
148
density-of-targets
density-of-targets
1
100
1
1
1
%
HORIZONTAL

MONITOR
5
515
95
560
N:o of targets
length list-of-target-values
17
1
11

CHOOSER
140
45
270
90
visualization?
visualization?
"ln" "decimal" "None"
2

MONITOR
195
380
285
425
Highest cost
[cost] of max-one-of techs [cost]
17
1
11

SLIDER
5
150
180
183
density-of-valleys
density-of-valleys
0
100
1
1
1
%
HORIZONTAL

MONITOR
500
380
557
425
NIL
ticks
17
1
11

PLOT
125
585
435
805
Repertoire
NIL
NIL
0.0
10.0
0.0
10.0
true
true
"" ""
PENS
"Active" 1.0 0 -16777216 true "" "plot count techs with [active-repertoire? = TRUE]"
"Total" 1.0 0 -7500403 true "" "plot count techs"

SLIDER
5
220
247
253
uncertainty-of-technology
uncertainty-of-technology
0
100
80
1
1
%
HORIZONTAL

MONITOR
5
615
95
660
Tgts reached
length list-of-reached-targets
17
1
11

MONITOR
290
380
367
425
Maxed out
count techs with [[product] of self = [cost] of self]
17
1
11

PLOT
100
430
340
585
Cost
Ticks
Cost
0.0
10.0
0.0
10.0
true
true
"set-plot-x-range 0 stop-at-tick" ""
PENS
"Max" 1.0 0 -16777216 true "" "plot max [cost] of techs"
"Average" 1.0 0 -7500403 true "" "plot mean [cost] of techs"
"Targets" 1.0 2 -2674135 true "" "if targets-reached-this-turn? = TRUE [plotxy ticks [cost] of tech last-tech-who-created]"

MONITOR
370
380
497
425
Actives maxed out
count techs with [active-repertoire? = TRUE and [product] of self = [cost] of self]
17
1
11

MONITOR
5
565
95
610
Tgts close
length list-of-almost-reached-targets
17
1
11

PLOT
100
815
445
965
Targets reached
Ticks
Targets
0.0
10.0
0.0
10.0
true
true
"" ""
PENS
"Targets reached" 1.0 0 -16777216 true "" "plot length list-of-reached-targets"
"Almost reached" 1.0 0 -7500403 true "" "plot length list-of-almost-reached-targets"
"Targets total" 1.0 0 -2674135 true "" "plot length list-of-target-values"

PLOT
705
430
865
580
Top tech
Ticks
Tech level
0.0
10.0
0.0
10.0
true
false
"set-plot-x-range 0 stop-at-tick" ""
PENS
"Highest tech reached" 1.0 0 -16777216 true "" "plot highest-tech-reached"

INPUTBOX
315
10
467
70
stop-at-tick
25000
1
0
Number

PLOT
345
430
505
580
Spread of techs
Users
N:o of techs
1.0
20.0
0.0
10.0
true
false
"" ""
PENS
"default" 1.0 1 -16777216 true "" "histogram [how-many-use-me] of techs"

MONITOR
5
665
95
710
Valleys
length list-of-valleys
17
1
11

SLIDER
5
255
245
288
uncertainty-of-needs
uncertainty-of-needs
0
100
80
1
1
%
HORIZONTAL

SLIDER
195
115
382
148
learning-curve-tech
learning-curve-tech
0
100
20
1
1
%
HORIZONTAL

SLIDER
190
150
382
183
learning-curve-need
learning-curve-need
0
100
20
1
1
%
HORIZONTAL

PLOT
510
430
700
580
Space of adjacent possible
NIL
NIL
0.0
10.0
0.0
10.0
true
false
"set-plot-x-range 0 stop-at-tick" ""
PENS
"Adjanced" 1.0 0 -16777216 true "" "plot length space-of-adjacent-possible"

CHOOSER
260
260
412
305
valley_effects
valley_effects
"product-level" "component-level"
1

PLOT
500
585
700
735
Created and failed
NIL
NIL
0.0
2.0
0.0
10.0
true
false
"" ""
PENS
"Created" 1.0 0 -10899396 true "" "plotxy 1 1"
"Failed" 1.0 0 -2674135 true "" ""

MONITOR
665
380
757
425
Top adjacent
max space-of-adjacent-possible
17
1
11

SWITCH
460
220
607
253
random-seed?
random-seed?
0
1
-1000

SLIDER
195
180
367
213
neighborhood-size
neighborhood-size
0
10
1
1
1
NIL
HORIZONTAL

INPUTBOX
475
10
710
70
dump_ticks
[25000]
1
0
String

SWITCH
315
75
447
108
data_dump?
data_dump?
0
1
-1000

SWITCH
475
75
647
108
dump_every_turn?
dump_every_turn?
1
1
-1000

INPUTBOX
620
220
690
280
seed
84047
1
0
Number

MONITOR
620
170
690
215
Seed used
seed-number
17
1
11

BUTTON
390
155
517
188
Load saved tgts
setup\nload-targets-and-valleys\nsetup-data-dump
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SWITCH
255
220
452
253
infinite-neighborhood?
infinite-neighborhood?
1
1
-1000

@#$#@#$#@
## WHAT IS IT?

ADDER models the evolution of complex technological systems from simple components. Technologies are represented by  their "products," or natural numbers (1,2,3,...,n). For example, "Tech 4" means a technology whose "product" is number 4. 

New technologies are created by combining (adding) existing technologies. For example, a Tech 7 could be composed of Techs 4 and 1 as follows: [4 + 4 - 1]. Under default settings, every new technology can be used as a component in  the future; therefore, once Tech 7 is created, a Tech 14 could be created by [7 + 7]. The rules by which technologies are selected and how many components a technology can have may vary depending on the simulation setting. 

## HOW IT WORKS

The model starts from a predefined set of "primitives" or primitive technologies, which are initialized in the Setup ("Creating a set of primitives"). Under default settings, there is one primitive, which has a product 1 and a cost of 1.

### Variables explained

#### Techs-own
* _product:_ the "product" of technology; what it produces.
* _cost:_ the cost of technology, calculated as a sum of cost of component technologies. By default, each primitive used has a cost of 1, so a tech [1 + 1 - 1] would have a product 2 and cost 3. 
* _distance-to-nearest-target:_ the distance to nearest target value. Either positive or negative.
* _nearest-peak:_ the target value that is closest to the tech's product.
* _components:_ a list of component technologies used, e.g. [4 3 1 0 0]. Zeros represent "no tech" and are there to allow for variable number of components (up to the Interface variable max-components; in the above example, max-components = 5). In the default version, this list remains unchanging from the creation of the technology. 
operators: a list of operators (+ or -) for each component. A tech's product is determined by adding components together with appropriate operators. To continue the above example, the tech might have a following list of operators: [1 1 -1 1 1]. Therefore, the tech's product would be 4 + 3 - 1 + 0 + 0 = 6. 
* _cost-of-components:_ a list for the cost of components used. This list is updated if component technologies are updated (see below).
* _contributor-tech-whos:_ a list of technologies (by [who] ID) which are being used as components in this technology. Used to establish relationship between technologies and in updating the cost of components.
* _who-uses-me:_ the same as above but in reverse; the [who]s of technologies that use this technology as a component.
* _how-many-use-me:_ The length of above list. For performance reasons, calculated for each tech only when the tech is updated (another option would be to use LENGTH reporter). 
* _tech-ID:_ somewhat redundant alternative ID for technology, may be deleted in later versions.
* _created:_ timestamp (tick) when the technology is created.
* _last-updated:_ timestamp (tick) when the technology has been last updated (ie. when its components have become cheaper).
* _active-repertoire?_ TRUE/FALSE variable, whether the technology is in active use ie. can be used as a component for future technologies.
* _debug:_ catch-all variable for debugging.

#### Globals:
* _next-tech-ID:_ a counter for next tech ID.
* _list-of-operators:_ a list of allowed operators. Set in Setup. By default, + and -.
* _list-of-target-values:_ list of "targets" that technologies try to satisfy (e.g. 223). Set up in Setup, "Creating the peaks" (and initialized in the beginning of the setup).
* _list-of-reached-targets:_ a list, originally initialized as empty, that is populated by target values that have been reached exactly. The list is updated in technology creation phase, if a viable technology that satisfies the target exactly is created.
* _list-of-almost-reached-targets:_ In the current version, a target is almost reached if a technology with a product closer to that target than any other (e.g. already reached target) is created. This list is then updated. When a target is reached, it is added to * list-of-reached-targets and removed from this list.
* _list-of-valleys:_ a list of technologies that are not viable and cannot be created.
* _list-of-techs-with-efficiency-maxed-out:_ a list of technologies that cannot be improved any further. In default model, this means technologies whose cost equals product (as the cheapest way to produce, say, 4 is with four primitives [1+1+1+1], which has a cost of 4). This list is updated at the technology creation procedure.
* _failed-techs:_ a counter that is advanced when no tech is created this turn. For statistics.
* _cumulative-replacements:_ a counter of cumulative replacements of technologies. For statistics.
* _list-of-replacements:_ For statistics, used to draw a histogram in Interface screen.
* _highest-tech-reached:_ the highest tech level (product) reached so far.
* _list-of-techlevels-found:_ a list of all products created so far.
* _size-of-active-repertoire:_ a counter for statistics, tracking the size of active repertoire.
* _viable-techs-created-this-turn?_ A TRUE/FALSE variable that is set in the tech creation phase, after a prospective tech is drawn but before actually creating it.
* _last-techlevel-created:_ the product of the last technology created.
* _last-tech-who-created:_ [who] of last tech created so far.
* _stop-called?_ TRUE/FALSE variable that can be used by procedures to call a stop to the simulation.
* _obsoleted-techs:_ a list of technologies that have become obsoleted this turn. Used in the update sequence.
* _obsolete-tech-who:_ Used in the update sequence to move the [who] of current obsoleted tech to the update procedure.
* _prospective-_ : a set of global variables used in the creation of new technologies. A new tech is first created as a "prospective" tech, and "committed" to a newly-created agent ("tech") only if it satisfies the conditions set at the beginning of the simulation (e.g. it's not a "valley" tech).

### The procedures explained

#### The Setup procedure:
1. clears all variables
2. sets a random seed
3. sets a selected visualization display (ln or decimal ie. no scaling)
4. initializes lists and variables; mostly empty lists, but this sets up the list-of-operators
5. creates a set of primitives
6. creates the targets and the valleys of the "landscape:"
 1. the number of targets depends on the density-of-targets variable and the maximum tech level (stops-at-tech-level).
 2. the number of valleys depends on the density-of-valleys variable, so that density-of-valleys + density-of-targets ≤ 100 %
7. resets ticks
8. prints timestamp and ADDER READY

#### The Go procedure:
1. checks if conditions are reached for the simulation to stop
advances the tick
2. clears variables viable-techs and obsoleted-techs to FALSE and []
updates active repertoire, according to selected update model:
    1. least-cost
    2. distance-to-target-and-cost
3. updates visualization, if necessary
4. sets statistics variable size-of-active-repertoire
5. creates new technology according to selected new tech creation model
6. calls a procedure to check whether techs need updating
7. clears who-uses-me variables from duplicates

### UPDATE SCHEMES

####update-active-repertoire-least-cost:

1. begins a foreach loop for a list of products of technologies, with duplicates removed
2. for each product, the cheapest technology is added to the active repertoire

#### update-active-repertoire-check-distance-to-target-and-cost:
1. begins a foreach loop for each of almost-reached targets
    1. for each of these, selects the tech that is closest to the target
      1. if several techs are equally close, selects the cheapest
2. begins a foreach loop for each reached target
  1. picks the technology which is exactly on the target and has the lowest cost


### TECHNOLOGY CREATION PROCEDURES

#### randomize-new-tech:
1. clears "prospective-" variables
2. picks 2 technologies from the active repertoire as first components (this is to prevent the creation of technologies with 0 or 1 components)
    1. picks random operators for these technologies
    2. saves cost data for the components
3. repeats until max-components - 2 is reached:
    1. picks a technology from active repertoire
    2. randomizes whether the component is added, or not (in which case a 0 is added)
    3. picks a random operator
    4. if a (non-zero) component is added: 
    1. saves cost data
    2. saves contributor-techs [who] to prospective-contributor-tech-whos list
  5. if no component is added (above randomizer gets a 0):
    1. saves a 0 to the list of component costs
    2. saves a "na" to the list of prospective contributor tech whos list (to retain the correct positions for costs, whos &c.)

3. determines the product: combines components with operators and adds them up

4. determines distance to target values:
    1. creates a distance-to-target-product-values list by comparing the product with target values
    2. creates a version of the above list with absolute values, so that <min> function works properly
    3. checks the position of minimum absolute value on the list
    4. uses this position to pick the proper distance to target value from the distance-to-target-product-values list
5. checks whether the created tech is "viable" (ie. allowed)
    1. checks against list of valleys
    2. checks against list of techs with efficiency maxed out (cost = product)
    3. checks whether the tech is negative or not, and whether negative techs are allowed
    4. checks the tech is an improvement to the existing situation
    5. if conditions are not satisfied, sets viable-techs-created-this-turn? and proceed-to-tech-creation? to FALSE

6. checks whether a tech can be created (proceed-to-tech-creation? = TRUE); if yes, calls the procedure create-next-tech

#### Creating next technology:
1. inherits variables from prospective-variables
3. sets last-techlevel-created as a product
4. sets up other variables
5. sets viable-techs-created-this-turn? to TRUE
6. checks whether new targets are to be added to list-of-almost-reached targets
7. checks whether new targets are to be added to the list-of-reached-targets
    1. if yes, removes the target from the list-of-almost-reached-targets
8. checks if the tech is now maxed out in efficiency (|product| = cost; absolute value to allow negative technologies if desired)
    1. if yes, adds to the list-of-techs-with-efficiency-maxed-out
9. checks if technology frontier (highest tech reached) advances
10. updates position in the visualization

### CHECKING WHETHER TO UPDATE TECHNOLOGIES
1. if viable-techs-created-this-turn? = TRUE:
    1. if there are existing technologies with the same tech level than the one created this turn:
      1. populates the list obsoleted-techs with techs that i) have the same tech level and ii) are more expensive than the last tech created
    2. begins a while loop until obsoleted-tech list is empty
      1. stores the current obsoleted-tech to a variable obsolete-tech-who
      2. calls for update-techs procedure

### UPDATING TECHNOLOGIES

#### update-techs-simple:

1. picks an agentset with the same product as the obsolete-tech-who. This set includes the obsoleted technology, as it's possible that there are no replacements but the technology has just gotten cheaper.
    1. out of these, picks the cheapest as a replacement
2. asks other techs:
    1. if the obsoleted-tech in question is used by them:
    2. determines the position in the component list
    3. replaces the [who] in contributor-tech-whos with the replacement's [who]
    4. replaces the cost in cost-of-components list with the replacement's cost
    5. recalculates cost
    6. sets last-updated variable with a timestamp
    7. removes the updated tech's [who] from the who-uses-me list of the obsoleted technology
    8. adds the updated tech's [who] to the who-uses-me list of the replacement technology
    9. updates position in the visualization
3. if someone is using this technology and the update reduces costs
    1. and if the update has reduced costs
    2. adds the updated tech's [who] to the list obsoleted-techs
4. once all the other technologies are checked, the obsoleted technology is removed from the obsoleted-techs list
5. if at least one technology has been updated, the list-of-replacements is updated with the information of how many techs were updated this turn (gives the cascade size)

## HOW TO USE IT

This section could explain how to use the model, including a description of each of the items in the interface tab.

## THINGS TO NOTICE

This section could give some ideas of things for the user to notice while running the model.

## THINGS TO TRY

This section could give some ideas of things for the user to try to do (move sliders, switches, etc.) with the model.

## EXTENDING THE MODEL

This section could give some ideas of things to add or change in the procedures tab to make the model more complicated, detailed, accurate, etc.

## NETLOGO FEATURES

This section could point out any especially interesting or unusual features of NetLogo that the model makes use of, particularly in the Procedures tab.  It might also point out places where workarounds were needed because of missing features.

## RELATED MODELS

This section could give the names of models in the NetLogo Models Library or elsewhere which are of related interest.

## CREDITS AND REFERENCES

This section could contain a reference to the model's URL on the web if it has one, as well as any other necessary credits or references.
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
0
Rectangle -7500403 true true 151 225 180 285
Rectangle -7500403 true true 47 225 75 285
Rectangle -7500403 true true 15 75 210 225
Circle -7500403 true true 135 75 150
Circle -16777216 true false 165 76 116

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 5.0.1
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
<experiments>
  <experiment name="experiment2-ECMS test" repetitions="1" runMetricsEveryStep="true">
    <setup>setup</setup>
    <go>go</go>
    <metric>seed-number</metric>
    <metric>count techs</metric>
    <metric>[product] of techs</metric>
    <metric>[cost] of techs</metric>
    <metric>length [components] of techs</metric>
    <metric>[how-many-use-me] of techs</metric>
    <metric>length space-of-adjacent-possible</metric>
    <metric>length list-of-reached-targets</metric>
    <metric>length list-of-almost-reached-targets</metric>
    <enumeratedValueSet variable="max-tech-level">
      <value value="20000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="learning-curve-need">
      <value value="80"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="debugging-mode?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="valley_effects">
      <value value="&quot;component-level&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="uncertainty-of-needs">
      <value value="0"/>
      <value value="80"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="stop-at-tick">
      <value value="1000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="uncertainty-of-technology">
      <value value="0"/>
      <value value="80"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="kill-negative-techs?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="learning-curve-tech">
      <value value="80"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="density-of-valleys">
      <value value="1"/>
      <value value="20"/>
      <value value="50"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="density-of-targets">
      <value value="1"/>
      <value value="20"/>
      <value value="50"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="visualization?">
      <value value="&quot;None&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="max-components">
      <value value="50"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="experiment3-ECMS test" repetitions="1" runMetricsEveryStep="true">
    <setup>setup</setup>
    <go>go</go>
    <metric>seed-number</metric>
    <metric>count techs</metric>
    <metric>[product] of techs</metric>
    <metric>[cost] of techs</metric>
    <metric>length [components] of techs</metric>
    <metric>[how-many-use-me] of techs</metric>
    <metric>length space-of-adjacent-possible</metric>
    <metric>length list-of-reached-targets</metric>
    <metric>length list-of-almost-reached-targets</metric>
    <enumeratedValueSet variable="max-tech-level">
      <value value="20000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="learning-curve-need">
      <value value="80"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="debugging-mode?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="valley_effects">
      <value value="&quot;component-level&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="uncertainty-of-needs">
      <value value="0"/>
      <value value="80"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="stop-at-tick">
      <value value="1000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="uncertainty-of-technology">
      <value value="0"/>
      <value value="80"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="kill-negative-techs?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="learning-curve-tech">
      <value value="80"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="density-of-valleys">
      <value value="1"/>
      <value value="20"/>
      <value value="40"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="density-of-targets">
      <value value="1"/>
      <value value="20"/>
      <value value="40"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="visualization?">
      <value value="&quot;None&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="max-components">
      <value value="50"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="experiment4-ECMS test" repetitions="5" runMetricsEveryStep="true">
    <setup>setup</setup>
    <go>go</go>
    <metric>seed-number</metric>
    <metric>count techs</metric>
    <metric>[product] of techs</metric>
    <metric>[cost] of techs</metric>
    <metric>length [components] of techs</metric>
    <metric>[how-many-use-me] of techs</metric>
    <metric>length space-of-adjacent-possible</metric>
    <metric>length list-of-reached-targets</metric>
    <metric>length list-of-almost-reached-targets</metric>
    <enumeratedValueSet variable="max-tech-level">
      <value value="20000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="random-seed?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="learning-curve-need">
      <value value="80"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="debugging-mode?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="valley_effects">
      <value value="&quot;component-level&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="uncertainty-of-needs">
      <value value="0"/>
      <value value="80"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="stop-at-tick">
      <value value="1000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="uncertainty-of-technology">
      <value value="0"/>
      <value value="80"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="kill-negative-techs?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="learning-curve-tech">
      <value value="80"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="density-of-valleys">
      <value value="1"/>
      <value value="20"/>
      <value value="40"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="density-of-targets">
      <value value="1"/>
      <value value="20"/>
      <value value="40"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="visualization?">
      <value value="&quot;None&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="max-components">
      <value value="50"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="experiment5-ECMS test with LHS" repetitions="1" runMetricsEveryStep="true">
    <setup>load-experiment-number
setup-parameters
setup</setup>
    <go>go</go>
    <final>save-experiment-number</final>
    <exitCondition>experiment-number = 100</exitCondition>
    <metric>seed-number</metric>
    <metric>density-of-targets</metric>
    <metric>density-of-valleys</metric>
    <metric>uncertainty-of-technology</metric>
    <metric>uncertainty-of-needs</metric>
    <metric>learning-curve-tech</metric>
    <metric>learning-curve-need</metric>
    <metric>count techs</metric>
    <metric>last-techlevel-created</metric>
    <metric>targets-reached-this-turn?</metric>
    <metric>who-of-last-tech-created</metric>
    <metric>[cost] of tech who-of-last-tech-created</metric>
    <metric>list [components] of tech who-of-last-tech-created</metric>
    <metric>list [contributor-tech-whos] of tech who-of-last-tech-created</metric>
    <metric>max [how-many-use-me] of techs</metric>
    <metric>[how-many-use-me] of techs</metric>
    <metric>length space-of-adjacent-possible</metric>
    <metric>length list-of-almost-reached-targets</metric>
    <metric>length list-of-target-values</metric>
    <metric>length list-of-valleys</metric>
    <enumeratedValueSet variable="max-components">
      <value value="50"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="kill-negative-techs?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="stop-at-tick">
      <value value="1500"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="debugging-mode?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="valley_effects">
      <value value="&quot;component-level&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="random-seed?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="max-tech-level">
      <value value="20000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="visualization?">
      <value value="&quot;None&quot;"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="experiment6-ECMS test" repetitions="5" runMetricsEveryStep="true">
    <setup>setup</setup>
    <go>go</go>
    <metric>seed-number</metric>
    <metric>density-of-targets</metric>
    <metric>density-of-valleys</metric>
    <metric>uncertainty-of-technology</metric>
    <metric>uncertainty-of-needs</metric>
    <metric>learning-curve-tech</metric>
    <metric>learning-curve-need</metric>
    <metric>count techs</metric>
    <metric>last-techlevel-created</metric>
    <metric>targets-reached-this-turn?</metric>
    <metric>who-of-last-tech-created</metric>
    <metric>[cost] of tech who-of-last-tech-created</metric>
    <metric>list [components] of tech who-of-last-tech-created</metric>
    <metric>list [contributor-tech-whos] of tech who-of-last-tech-created</metric>
    <metric>max [how-many-use-me] of techs</metric>
    <metric>[how-many-use-me] of techs</metric>
    <metric>length space-of-adjacent-possible</metric>
    <metric>length list-of-almost-reached-targets</metric>
    <metric>length list-of-target-values</metric>
    <metric>length list-of-valleys</metric>
    <enumeratedValueSet variable="max-tech-level">
      <value value="20000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="random-seed?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="learning-curve-need">
      <value value="80"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="debugging-mode?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="valley_effects">
      <value value="&quot;component-level&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="uncertainty-of-needs">
      <value value="0"/>
      <value value="80"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="stop-at-tick">
      <value value="1000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="uncertainty-of-technology">
      <value value="0"/>
      <value value="80"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="kill-negative-techs?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="learning-curve-tech">
      <value value="80"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="density-of-valleys">
      <value value="1"/>
      <value value="20"/>
      <value value="40"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="density-of-targets">
      <value value="1"/>
      <value value="20"/>
      <value value="40"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="visualization?">
      <value value="&quot;None&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="max-components">
      <value value="50"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="experiment" repetitions="1" runMetricsEveryStep="true">
    <setup>setup
load-targets-and-valleys
setup-data-dump</setup>
    <go>go</go>
    <enumeratedValueSet variable="max-components">
      <value value="50"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="valley_effects">
      <value value="&quot;component-level&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="learning-curve-tech">
      <value value="80"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="debugging-mode?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="seed">
      <value value="35440"/>
      <value value="40508"/>
      <value value="42509"/>
      <value value="75881"/>
      <value value="84047"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="kill-negative-techs?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="stop-at-tick">
      <value value="1000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="uncertainty-of-needs">
      <value value="80"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="random-seed?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="max-tech-level">
      <value value="20000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="density-of-valleys">
      <value value="50"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="uncertainty-of-technology">
      <value value="0"/>
      <value value="10"/>
      <value value="20"/>
      <value value="30"/>
      <value value="40"/>
      <value value="50"/>
      <value value="60"/>
      <value value="70"/>
      <value value="80"/>
      <value value="90"/>
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="neighborhood-size">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="learning-curve-need">
      <value value="10"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="data_dump?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="dump_ticks">
      <value value="&quot;[0 100 200 300 400 500 600 700 800 900 1000]&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="density-of-targets">
      <value value="50"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="visualization?">
      <value value="&quot;None&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="dump_every_turn?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="infinite-neighborhood?">
      <value value="true"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="Chap4_Learn80" repetitions="10" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <metric>seed-number</metric>
    <metric>density-of-targets</metric>
    <metric>density-of-valleys</metric>
    <metric>uncertainty-of-technology</metric>
    <metric>uncertainty-of-needs</metric>
    <metric>learning-curve-tech</metric>
    <metric>learning-curve-need</metric>
    <metric>count techs</metric>
    <metric>last-techlevel-created</metric>
    <metric>targets-reached-this-turn?</metric>
    <metric>who-of-last-tech-created</metric>
    <metric>[cost] of tech who-of-last-tech-created</metric>
    <metric>list [components] of tech who-of-last-tech-created</metric>
    <metric>list [contributor-tech-whos] of tech who-of-last-tech-created</metric>
    <metric>max [how-many-use-me] of techs</metric>
    <metric>[how-many-use-me] of techs</metric>
    <metric>length space-of-adjacent-possible</metric>
    <metric>length list-of-almost-reached-targets</metric>
    <metric>length list-of-target-values</metric>
    <metric>length list-of-valleys</metric>
    <enumeratedValueSet variable="max-tech-level">
      <value value="1000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="data_dump?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="dump_every_turn?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="random-seed?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="learning-curve-need">
      <value value="80"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="debugging-mode?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="valley_effects">
      <value value="&quot;component-level&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="uncertainty-of-needs">
      <value value="0"/>
      <value value="20"/>
      <value value="80"/>
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="stop-at-tick">
      <value value="25000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="uncertainty-of-technology">
      <value value="0"/>
      <value value="20"/>
      <value value="80"/>
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="kill-negative-techs?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="learning-curve-tech">
      <value value="80"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="density-of-valleys">
      <value value="20"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="density-of-targets">
      <value value="1"/>
      <value value="25"/>
      <value value="50"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="visualization?">
      <value value="&quot;None&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="max-components">
      <value value="50"/>
    </enumeratedValueSet>
  </experiment>
</experiments>
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 1.0 0.0
0.0 1 1.0 0.0
0.2 0 1.0 0.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
1
@#$#@#$#@
